﻿using System.Windows.Forms;
using ESRI.ArcGIS.Carto;
using ESRI.ArcGIS.Desktop.AddIns;
using ESRI.ArcGIS.Display;
using ICommandItem = ESRI.ArcGIS.Framework.ICommandItem;
using ESRI.ArcGIS.Geometry;
using orbitgt.mapping3d.sdk.viewer;

namespace plugin_3dm_arcmap
{
    /// <summary>
    /// Class ToolPickLocation defines a tool that lets the user pick a position on a map view and send it to the 3D Mapping viewer
    /// </summary>
    internal class ToolPickLocation : Tool
    {
        static public ToolPickLocation INSTANCE = null;

        internal bool pickPositionMode = false; // either "pick-position" or "focus" (default) mode

        public ToolPickLocation()
        {
            INSTANCE = this;
            this.Cursor = Cursors.Cross;
        }

        internal void startPickPosition()
        {
            this.pickPositionMode = true;
            ICommandItem commandItem = ArcMap.Application.Document.CommandBars.Find("plugin_3dm_arcmap_ToolPickLocation", false);
            if (commandItem != null) ArcMap.Application.CurrentTool = commandItem;
        }

        internal void stopPickPosition()
        {
            ICommandItem commandItem = ArcMap.Application.Document.CommandBars.Find("plugin_3dm_arcmap_ToolPickLocation", false);
            if ((commandItem != null) && (ArcMap.Application.CurrentTool==commandItem)) ArcMap.Application.CurrentTool = null;
            this.pickPositionMode = false;
        }

        protected override void OnUpdate()
        {
            this.Cursor = Cursors.Cross;
        }

        protected override void OnActivate()
        {
        }

        protected override bool OnDeactivate()
        {
            return base.OnDeactivate();
        }

        protected override void OnMouseDown(MouseEventArgs e)
        {
            // we should have a viewer
            if ((PluginArcMap.INSTANCE != null) && PluginArcMap.INSTANCE.isReady())
            {
                // get the click position in the viewer CRS
                IDisplay display = (PluginArcMap.INSTANCE.getMap() as IActiveView).ScreenDisplay;
                IPoint point = display.DisplayTransformation.ToMapPoint(e.X,e.Y);
                point.Project(MapObjects.VIEWER_CRS);
                // make the viewer focus to the new position
                Viewer3DM viewer = PluginArcMap.INSTANCE.getViewer();
                Coordinate position = Coordinate.fromXY("" + MapObjects.VIEWER_CRS_CODE, point.X, point.Y);
                if (this.pickPositionMode) viewer.setPickPosition(position);
                else viewer.setFocusPosition(position);
            }
            // clear the pick-position mode
            this.pickPositionMode = false;
        }
    }
}
